#include "animation.h"

//32 is about 99% scale
#define MAX_SCALE 32

typedef enum state {
	DECREASING = 10,
	INCREASING = 20,
	IDLE = 30
} state;

sprite* p_from;
sprite* p_to;

state p_lastState;
state p_state;
int p_currentScale;

void animation_init() {
	p_lastState = IDLE;
	p_state = IDLE;
}

void p_changeStateTo( state newState ) {
	//p_lastState = p_state;
	p_state = newState;
}

void animation_start( sprite* fromSprite, sprite* toSprite ) {
	if( animation_isRunning() ) {
		return;
	}
	
	p_from = fromSprite;
	p_to = toSprite;
	
	p_changeStateTo( DECREASING );
	p_currentScale = MAX_SCALE;
}

int animation_isRunning() {
	return p_state != IDLE;
}

int animation_hasJustFinished() {
	return p_lastState != IDLE && p_state == IDLE;
}

void p_applyScale( sprite* which ) {
	which->scaled = 1;
	which->hscale = p_currentScale;
	which->invisible = 0;
}


void p_handleIncreasing() {
	p_currentScale++;
	
	// is it large enough?
	if( p_currentScale >= MAX_SCALE ) {
		p_to->scaled = 0;
		p_changeStateTo( IDLE );
		
		return;
	}
	
	p_applyScale( p_to );
}


void p_handleDecreasing() {
	p_currentScale--;
	
	// has it decreased to nothingness? :)
	if( p_currentScale < 0 ) {
		p_changeStateTo( INCREASING );
		p_from->scaled = 0;
		p_from->invisible = 1;
		
		return;
	}
	
	p_applyScale( p_from );
}

void animation_advance() {
	p_lastState = p_state;
	
	if( !animation_isRunning() ) {
		return;
	}
	
	if( p_state == DECREASING ) {
		p_handleDecreasing();
	}
	
	if( p_state == INCREASING ) {
		p_handleIncreasing();
	}
}