#include "utilities.h"

display* p_display;
joypad_state* p_state;
FILE* p_console;

long p_oldInputValue;
long p_inputValue;

void input_read() {
	p_oldInputValue = p_inputValue;
	
	read_joypad_state( p_state );
	p_inputValue = p_state->j1;
}

long input_isTapped( long which ) {
	long wasDownLastFrame = p_oldInputValue & which;
	long isDownThisFrame = p_inputValue & which;
	
	return isDownThisFrame && !wasDownLastFrame;
}

void input_init() {
	p_state = malloc( sizeof( joypad_state ) );
	
	input_read();
	input_read();
}

void game_init() {
	TOMREGS->vmode = RGB16 | CSYNC | BGEN | PWIDTH4 | VIDEN;

	init_interrupts();
	init_display_driver();
	
	p_display = new_display( 0 );
	// position of display relative to (0, 0)
	p_display-> x = 19;
	p_display-> y = 7;
	
	show_display( p_display );
		
	//p_console = open_console( game_getDisplay(), 0, 0, 0 );
	//p_console = open_custom_console( game_getDisplay(), 0, 0, 0, 20, 3, 2 );
	#define CONSOLE_COLS 10
	#define CONSOLE_ROWS 1
	#define CONSOLE_X 240
	#define CONSOLE_Y 200
	p_console = open_custom_console( 
		game_getDisplay(), 
		CONSOLE_X, 
		CONSOLE_Y, 
		0, 
		CONSOLE_COLS, 
		CONSOLE_ROWS, 
		2
		);
}

display* game_getDisplay() {
	return p_display;
}

FILE* game_getConsole() {
	return p_console;
}

sprite* game_makeSprite(
	int width, // width in pixels so that the width in bytes is a multiple of 8
	int height, 
	phrase* data // address of the graphical data: it should be phrase aligned
	) {
	
	int layer = 1;
	
	int hotspotX = width / 2;
	int hotspotY = height / 2;
	
	// since hotspot is in the centre, by setting location X and Y to the same
	// coordinates as the hotspot, the sprite is initialized at display (0, 0) essentially
	sprite* sprite = new_sprite( width, height, hotspotX, hotspotY, DEPTH16, data);
	sprite->use_hotspot = 1;
	sprite->hx = hotspotX;
	sprite->hy = hotspotY;
	
	attach_sprite_to_display_at_layer( sprite, game_getDisplay(), layer );
	
	return sprite;
}

void shuffleInPlace( int* data, int count ) {
	#define NUM_ITERATIONS 2000
	unsigned int i, temp, firstIndex, secondIndex;
	
	for( i = 0; i < NUM_ITERATIONS; i++ ) {
		firstIndex = random( count );
		secondIndex = random( count );
				
		temp = data[secondIndex];
		data[secondIndex] = data[firstIndex];
		data[firstIndex] = temp;
	}
}

unsigned int random( unsigned int max ) {
	return (unsigned int) rand() % max;
}